@@ -8,23 +8,31 @@ class AutoSolve:
     ERROR = "error"
     STATUS = "status"
 
-    EXCHANGE = "exchanges.autosolve"
+    EXCHANGE = "exchanges.autosolve.direct"
+    CANCEL_EXCHANGE = "exchanges.autosolve.fanout"
     SENDER = "token.request"
+    CANCEL = "token.request.cancel"
     RECEIVER = "token.response"
     HEARTBEAT = 10
 
-    def __init__(self, access_token, api_key, client_key, receiver_function, debug):
-        self.access_token = access_token
-        self.api_key = api_key
-        self.client_key = client_key
-        self.receiver_function = receiver_function
+    def __init__(self, params):
+        self.access_token = params["access_token"]
+        self.api_key = params["api_key"]
+        self.client_key = params["client_key"]
+        self.receiver_function = params["receiver_function"]
+        self.cancel_function = params["cancel_function"]
         self.username = ""
         self.vhost = ""
-        self.routing_key = ""
+        self.routing_key = self.api_key.replace("-", "")
+        self.routing_key_receiver = ""
+        self.routing_key_cancel = ""
+        self.receiver_queue_name = "queues.token.response." + self.routing_key
+        self.cancel_queue_name = "queues.token.request.cancel." + self.routing_key
         self.connection = None
         self.channel = None
+        self.cancel_channel = None
         self.consumer_thread = None
-        self.debug = debug
+        self.debug = params["debug"]
         self.ready = False
         self.connected = False
         self.message_backlog = []
@@ -39,7 +47,7 @@ class AutoSolve:
 
     def start_connection_process(self):
         if self.check_input_values():
-            self.create_routing_key()
+            self.create_routing_keys()
             self.establish_rabbit_session()
 
     def establish_rabbit_session(self):
@@ -74,21 +82,30 @@ class AutoSolve:
 
             self.log("Creating channel", self.STATUS)
             self.channel = self.connection.channel()
+            self.cancel_channel = self.connection.channel()
+
             self.channel.confirm_delivery()
-            self.log("Channel created", self.SUCCESS)
+            self.cancel_channel.confirm_delivery()
+            self.log("Channels created", self.SUCCESS)
 
             self.log("Declaring exchange", self.STATUS)
             self.channel.exchange_declare(exchange=self.EXCHANGE, exchange_type='direct', durable=True)
-            queue_result = self.channel.queue_declare("", durable=True, auto_delete=True)
-            queue_name = queue_result.method.queue
+            self.cancel_channel.exchange_declare(exchange=self.CANCEL_EXCHANGE, exchange_type='fanout', durable=True)
 
-            self.log("Queue " + queue_name + " declared", self.SUCCESS)
+            self.channel.queue_declare(self.receiver_queue_name, durable=True, auto_delete=False)
+
+            self.log("Queue " + self.receiver_queue_name + " declared", self.SUCCESS)
 
             self.log("Binding queue to exchange", self.STATUS)
-            self.channel.queue_bind(queue=queue_name, exchange=self.EXCHANGE, routing_key=self.routing_key)
+            self.channel.queue_bind(queue=self.receiver_queue_name,
+                                    exchange=self.EXCHANGE, routing_key=self.routing_key_receiver)
             self.log("Queue binded to exchange " + self.EXCHANGE, self.SUCCESS)
 
-            self.channel.basic_consume(queue=queue_name, auto_ack=True, on_message_callback=self.on_message)
+            self.channel.basic_consume(queue=self.receiver_queue_name, auto_ack=True, on_message_callback=self.on_receiver_message)
+
+            if self.cancel_function is not None:
+                self.register_cancel_queue()
+
             self.log("Beginning message consumption", self.SUCCESS)
             self.ready = True
             self.connected = True
@@ -101,12 +118,26 @@ class AutoSolve:
     def close_connection(self):
         self.connection.close()
 
-    def on_message(self, channel, method_frame, header_frame, body):
+    def register_cancel_queue(self):
+        self.channel.queue_declare(self.cancel_queue_name, durable=True, auto_delete=False)
+        self.log("Queue " + self.cancel_queue_name + " declared", self.SUCCESS)
+        self.channel.queue_bind(queue=self.cancel_queue_name,
+                                exchange=self.EXCHANGE, routing_key=self.routing_key_cancel)
+        self.channel.basic_consume(queue=self.cancel_queue_name, auto_ack=True,
+                                   on_message_callback=self.on_cancel_message)
+
+    def on_receiver_message(self, channel, method_frame, header_frame, body):
         json_string = "".join(chr(x) for x in body)
         json_object = json.loads(json_string)
         self.log("Message Received: " + json_string, self.SUCCESS)
         self.receiver_function(json_object)
 
+    def on_cancel_message(self, channel, method_frame, header_frame, body):
+        json_string = "".join(chr(x) for x in body)
+        json_object = json.loads(json_string)
+        self.log("Message Received: " + json_string, self.SUCCESS)
+        self.cancel_function(json_object)
+
     def handle_connection_error(self):
         self.connected = False
         while True:
@@ -123,31 +154,45 @@ class AutoSolve:
         else:
             self.log("Connection attempts unsuccessful", self.ERROR)
 
-    def send(self, message):
+    def send(self, message, route, exchange):
         message_string = json.dumps(message)
         byte_string = message_string.encode()
-        return self.channel.basic_publish(exchange=self.EXCHANGE,
-                                          routing_key=self.SENDER,
+        return self.channel.basic_publish(exchange=exchange,
+                                          routing_key=route,
                                           body=byte_string)
 
     def send_token(self, message):
-        self.log("Sending message for task: " + message['taskId'], self.STATUS)
+        self.log("Sending request message for task: " + message['taskId'], self.STATUS)
         try:
-            send_result = self.send(message)
+            send_result = self.send(message, self.SENDER, self.EXCHANGE)
             if send_result is None:
                 self.log("Message with TaskId: " + message['taskId'] + " sent successfully", self.SUCCESS)
         except Exception as e:
             self.log(e, self.WARNING)
-            self.resend(message)
+            self.resend(message, self.SENDER, self.EXCHANGE)
 
-    def resend(self, message):
+    def cancel_request(self, taskId):
+        message = {"taskId" : taskId, "requireResponse" : False}
+        if self.cancel_function is not None:
+            message["requireResponse"] = True
+
+        self.log("Sending cancel message for task: " + message['taskId'], self.STATUS)
+        try:
+            send_result = self.send(message, self.CANCEL, self.CANCEL_EXCHANGE)
+            if send_result is None:
+                self.log("Message with TaskId: " + message['taskId'] + " sent successfully", self.SUCCESS)
+        except Exception as e:
+            self.log(e, self.WARNING)
+            self.resend(message, self.CANCEL, self.CANCEL_EXCHANGE)
+
+    def resend(self, message, route, exchange):
         time.sleep(5)
-        result = self.send(message)
+        result = self.send(message, route, exchange)
         if result is None:
             self.log("Resend attempt successful", self.SUCCESS)
         else:
             self.log("Attempt to resend after wait unsuccessful. Pushing message to backlog", self.WARNING)
-            self.add_message_to_backlog(message)
+            self.add_message_to_backlog({"message" : message, "route" : route, "exchange" : exchange})
 
     def add_message_to_backlog(self, message):
         self.message_backlog.append(message)
@@ -156,7 +201,7 @@ class AutoSolve:
         self.log("Sending messages from backlog", self.STATUS)
         message_cache = self.message_backlog
         for message in message_cache:
-            result = self.send(message)
+            result = self.send(message["message"], message["route"], message["exchange"])
             if result is None:
                 self.message_backlog.remove(message)
 
@@ -194,8 +239,9 @@ class AutoSolve:
         response = requests.get(url)
         return response.status_code
 
-    def create_routing_key(self):
-        self.routing_key = self.api_key.replace("-", "") + "." + self.RECEIVER
+    def create_routing_keys(self):
+        self.routing_key_receiver = self.RECEIVER + "." + self.routing_key
+        self.routing_key_cancel = self.CANCEL + "." + self.routing_key
 
     def log(self, message, type):
         if type == self.ERROR:
